<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;
use App\Account;
use App\Customer;
use App\Historique;
use Carbon\Carbon;


class AccountController extends Controller
{
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index()
    {
        $accounts = Account::all(); //Get all clients

        return view('accounts.index')->with('accounts', $accounts);
    }

    public function getCustomer()
    {
        $customers = Customer::all(); //Get all clients

        return view('accounts.checkcustomer', compact('customers'));
    }

    public function balance()
    {
        return view('accounts.checkbalance');
    }

    public function getBalance()
    {
       return view('accounts.getbalance');
    }

    public function postBalance(Request $request)
    {
        $num_account = $request->input('num_account');

        $accounts = Account::where('num_account', '=', $num_account)
                            ->get();
        
        /* if($account){
            return view('accounts.getbalance', compact('account'));
        }else{
            return view('accounts.getbalance')->with('error', 'Numéro de Compte Introuvable');
        } */

        //dd($account);

        if ($accounts) {
            return view('accounts.getbalance', compact('accounts'));
        }

        return view('accounts.getbalance')->withMessage('error');
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
        //return view('accounts.create');
    }

    public function add($id)
    {
        $customer = Customer::findOrFail($id);

        //$date_now = Carbon::now()->toDateString();

        //$date = $deposit->created_at->toDateString();

        return view('accounts.create', compact('customer'));
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        $this->validate($request, [
            'name' => 'required|max:120',
            'firstname' => 'required|max:120',
            'email' => 'required|email',
            'phone_number' => 'required',
            'address' => 'required',
            'profession' => 'required',
            'balance' => 'required',
        ]);

        $account = new Account();
        $account->owner_id = $request->input('customer_id');
        $account->owner_name = $request->input('name');
        $account->owner_firstname = $request->input('firstname');
        $account->owner_email = $request->input('email');
        $account->owner_phone = $request->input('phone_number');
        $account->owner_address = $request->input('address');
        $account->owner_profession = $request->input('profession');
        $account->balance = $request->input('balance');
        $account->status = 0;
        $account->user_id = auth()->user()->id;

        $dt = Carbon::now();
        Carbon::parse($dt);

        // These getters specifically return integers, ie intval()
        $code = intval($dt->year.''.$dt->month.''.$dt->day); 
        $account->num_account = $code;
        $random = substr(str_shuffle("0123456789"), 0, 5);

        $historique = new Historique();
        $historique->action = 'Create';
        $historique->table = 'Account';
        $historique->user_id = auth()->user()->id;
        $account->save();

        $account = Account::findOrFail($account->id);
        $account->num_account = $code.$account->id.$random;
        $account->save();
        $historique->save();
        
        //Redirect to the accounts.index view and display message
        return redirect()->route('accounts.index')
            ->with('success',
             'Nouveau Compte ajouté avec succès.');
    }

    /**
     * Display the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function show($id)
    {
        $account = Account::findOrFail($id); //Find account of id = $id

        return view('accounts.show', compact('account'));
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function edit($id)
    {
        $account= Account::findOrFail($id); //Find account of id = $id

        return view('accounts.edit', compact('account'));
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request, $id)
    {
        $this->validate($request, [
            'owner_name' => 'required|max:120',
            'owner_firstname' => 'required|max:120',
            'owner_email' => 'required|email',
            'owner_phone' => 'required',
            'owner_address' => 'required',
            'owner_profession' => 'required',
            'balance' => 'required',
        ]);

        $account = Account::findOrFail($id);
        $account->owner_id = $request->input('owner_id');
        $account->owner_name = $request->input('owner_name');
        $account->owner_firstname = $request->input('owner_firstname');
        $account->owner_email = $request->input('owner_email');
        $account->owner_phone = $request->input('owner_phone');
        $account->owner_address = $request->input('owner_address');
        $account->owner_profession = $request->input('owner_profession');
        $account->balance = $request->input('balance');
        $account->status = 0;
        $account->user_id = auth()->user()->id;

        $historique = new Historique();
        $historique->action = 'Update';
        $historique->table = 'Package';
        $historique->user_id = auth()->user()->id;
        $account->save();

        //Redirect to the accounts.index view and display message
        return redirect()->route('accounts.index')
            ->with('success',
            'Compte '.$account->num_account.' édité');
        
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function destroy($id)
    {
        $account = Account::findOrFail($id);

        $account->delete();

        return redirect()->route('accounts.index')
            ->with('success',
             'Compte Bancaire supprimé avec succès');
    }
}
