<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;
use App\Category;
use App\Historique;

class CategoryController extends Controller
{
    public function __construct()
    {
        $this->middleware(['auth', 'isAdmin']); //isAdmin middleware lets only users with a //specific permission permission to access these resources
    }
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index()
    {
        $categories = Category::all();

        return view('categories.index')->with('categories', $categories);
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
        return view('categories.create');
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        $this->validate($request, [
            'title' => 'required|max:100',
            'description' => 'required',
            //'cover_image' =>'image|nullable|max:1999',
        ]);

        $category = new Category();
        $category->title = $request['title'];
        $category->description = $request['description'];

        $historique = new Historique();
        $historique->action = 'create';
        $historique->table = 'Category';
        $historique->user_id = auth()->user()->id;

        $category->save();
        $historique->save();

        return redirect('categories')->with('success', 'Categorie créé');
    }

    /**
     * Display the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function show($id)
    {
        $category = Category::findOrFail($id); //Find categoryof id = $id

        return view('categories.show', compact('category'));
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function edit($id)
    {
        $category = Category::findOrFail($id); //Find categoryof id = $id

        return view('categories.edit', compact('category'));
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request, $id)
    {
        $this->validate($request, [
            'title' => 'required|max:100',
            'description' => 'required',
            //'cover_image' =>'image|nullable|max:1999',
        ]);

        $category = Category::findOrFail($id);
        $category->title = $request->input('title');
        $category->description = $request->input('description');

        $historique = new Historique();
        $historique->action = 'Update';
        $historique->table = 'Category';
        $historique->user_id = auth()->user()->id;

        $category->save();
        $historique->save();

        return redirect()->route('categories.index')
            ->with('success', 'Catégorie, '.$category->title.' édité');
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function destroy($id)
    {
        $category = Category::findOrFail($id);
        $category->delete();

        return redirect()->route('categories.index')
            ->with('success',
             'Categorie supprimé avec succès');
    }
}
