<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;
use App\Loan;
use App\Account;
use App\Customer;
use App\Historique;
use App\Category;

class LoanController extends Controller
{
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index()
    {
        $loans = Loan::all(); //Get all loans

        return view('loans.index')->with('loans', $loans);
    }

    public function getAccount()
    {
        $accounts = Account::all(); //Get all clients

        return view('loans.checkaccount', compact('accounts'));
    }

    public function add($id)
    {
        $account = Account::findOrFail($id);

        //$date_now = Carbon::now()->toDateString();

        //$date = $deposit->created_at->toDateString();

        $categories = Category::pluck('title', 'id');

        return view('loans.create', compact('account', 'categories'));
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
        //
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        $this->validate($request, [
            'applicant_name' => 'required|max:120',
            'applicant_firstname' => 'required|max:120',
            'applicant_email' => 'required|email',
            'applicant_phone' => 'required',
            'applicant_address' => 'required',
            'applicant_profession' => 'required',
            'loan_amount' => 'required',
            'rate' => 'required',
        ]);

        $loan = new Loan();
        $loan->applicant_id = $request->input('applicant_id');
        $loan->account_id = $request->input('account_id');
        $loan->applicant_numaccount = $request->input('applicant_numaccount');
        $loan->applicant_name = $request->input('applicant_name');
        $loan->applicant_firstname = $request->input('applicant_firstname');
        $loan->applicant_email = $request->input('applicant_email');
        $loan->applicant_phone = $request->input('applicant_phone');
        $loan->applicant_address = $request->input('applicant_address');
        $loan->applicant_profession = $request->input('applicant_profession');
        $loan->loan_amount = $request->input('loan_amount');
        $loan->rate = $request->input('rate');
        $loan->interest = ($loan->loan_amount * $loan->rate)/100;
        $loan->category_id = $request->input('category_id');

        $category = Category::findOrFail($loan->category_id);

        $loan->category_title = $category->title;
        $loan->status = 0;
        $loan->user_id = auth()->user()->id;

        $category = Category::findOrFail($loan->category_id);

        $loan->category_title = $category->title;
        $loan->status = 0;
        $loan->user_id = auth()->user()->id;

        $account = Account::findOrFail($loan->account_id);
        $account->balance += $loan->loan_amount;
        $account->save();

        $historique = new Historique();
        $historique->action = 'Create';
        $historique->table = 'Loan';
        $historique->user_id = auth()->user()->id;
        $loan->save();

        //Redirect to the loans.index view and display message
        return redirect()->route('loans.index')
            ->with('success',
             'Prêt ajouté avec succès.');

    }

    /**
     * Display the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function show($id)
    {
        $loan = Loan::findOrFail($id); //Find loan of id = $id

        return view('loans.show', compact('loan'));
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function edit($id)
    {
        $loan = Loan::findOrFail($id); //Find loan of id = $id

        $categories = Category::pluck('title', 'id');

        return view('loans.edit', compact('loan','categories'));
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request, $id)
    {
        $this->validate($request, [
            'applicant_name' => 'required|max:120',
            'applicant_firstname' => 'required|max:120',
            'applicant_email' => 'required|email',
            'applicant_phone' => 'required',
            'applicant_address' => 'required',
            'applicant_profession' => 'required',
            'loan_amount' => 'required',
            'rate' => 'required',
        ]);

        $loan = Loan::findOrFail($id);

        $old_amount = $loan->loan_amount;

        $loan->applicant_id = $request->input('applicant_id');
        $loan->account_id = $request->input('account_id');
        $loan->applicant_numaccount = $request->input('applicant_numaccount');
        $loan->applicant_name = $request->input('applicant_name');
        $loan->applicant_firstname = $request->input('applicant_firstname');
        $loan->applicant_email = $request->input('applicant_email');
        $loan->applicant_phone = $request->input('applicant_phone');
        $loan->applicant_address = $request->input('applicant_address');
        $loan->applicant_profession = $request->input('applicant_profession');
        $loan->loan_amount = $request->input('loan_amount');
        $loan->rate = $request->input('rate');

        $loan->interest = ($loan->loan_amount * $loan->rate)/100;

        $loan->category_id = $request->input('category_id');

        $category = Category::findOrFail($loan->category_id);

        $loan->category_title = $category->title;
        $loan->status = 0;
        $loan->user_id = auth()->user()->id;
        
        $account = Account::findOrFail($loan->account_id);
        $balance = $account->balance - $old_amount;
        $account->balance = $balance + $loan->loan_amount;
        $account->save();
        

        $historique = new Historique();
        $historique->action = 'Update';
        $historique->table = 'Loan';
        $historique->user_id = auth()->user()->id;
        $loan->save();

        //Redirect to the accounts.index view and display message
        return redirect()->route('loans.index')
            ->with('success',
             'Prêt édité avec succès.');
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function destroy($id)
    {
        $loan = Loan::findOrFail($id);

        $loan->delete();

        return redirect()->route('loans.index')
            ->with('success',
             'Prêt supprimé avec succès');
    }
}
