<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;
use App\Transaction;
use App\Account;
use App\Customer;
use App\Historique;
use Carbon\Carbon;

class TransactionController extends Controller
{
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index()
    {
        $transactions = Transaction::all(); //Get all transactions

        return view('transactions.index')->with('transactions', $transactions);
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
        return view('transactions.create');
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        $this->validate($request, [
            'sender_numAccount' => 'required|max:120',
            'receiver_numAccount' => 'required|max:120',
            'transaction_amount' => 'required',
        ]);

        $code1 = $request->input('sender_numAccount');
        $code2 = $request->input('receiver_numAccount');
        $account1 = Account::where('num_account', '=', $code1)
                            ->first();

        $account2 = Account::where('num_account', '=', $code2)
                            ->first();

        $transaction= new Transaction();
        $transaction->sender_id = $account1->owner_id;
        $transaction->sender_accountId = $account1->id;
        $transaction->sender_numAccount = $request->input('sender_numAccount');
        $transaction->sender_name = $account1->owner_name;
        $transaction->sender_firstname = $account1->owner_firstname;
        $transaction->sender_email = $account1->owner_email;
        $transaction->sender_phone = $account1->owner_phone;
        $transaction->sender_address = $account1->owner_address;
        $transaction->sender_profession = $account1->owner_profession;

        $transaction->receiver_id = $account2->owner_id;
        $transaction->receiver_accountId = $account2->id;
        $transaction->receiver_numAccount = $request->input('receiver_numAccount');
        $transaction->receiver_name = $account2->owner_name;
        $transaction->receiver_firstname = $account2->owner_firstname;
        $transaction->receiver_email = $account2->owner_email;
        $transaction->receiver_phone = $account2->owner_phone;
        $transaction->receiver_address = $account2->owner_address;
        $transaction->receiver_profession = $account2->owner_profession;

        $transaction->transaction_amount = $request->input('transaction_amount');
        $transaction->status = 0;
        $transaction->user_id = auth()->user()->id;

        $account_receiver = Account::findOrFail($transaction->receiver_accountId);
        $account_receiver->balance += $transaction->transaction_amount;
        $account_receiver->save();

        $account_sender = Account::findOrFail($transaction->sender_accountId);
        $account_sender->balance -= $transaction->transaction_amount;
        $account_sender->save();

        $dt = Carbon::now();
        Carbon::parse($dt);

        // These getters specifically return integers, ie intval()
        $code = intval($dt->year.''.$dt->month.''.$dt->day); 
        $transaction->code_transaction = $code;
        $random = substr(str_shuffle("0123456789"), 0, 5);

        $historique = new Historique();
        $historique->action = 'Create';
        $historique->table = 'TRANSACTION';
        $historique->user_id = auth()->user()->id;
        $transaction->save();

        $transaction = Transaction::findOrFail($transaction->id);
        $transaction->code_transaction = $code.$transaction->id.$random;
        $transaction->save();
        $historique->save();
        
        //Redirect to the accounts.index view and display message
        return redirect()->route('transactions.index')
            ->with('success','Nouvel Transaction effectué avec succès.');
    }

    /**
     * Display the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function show($id)
    {
        $transaction = Transaction::findOrFail($id); //Find transaction of id = $id

        return view('transactions.show', compact('transaction'));
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function edit($id)
    {
        $transaction = Transaction::findOrFail($id); //Find  transaction of id = $id

        return view('transactions.edit', compact('transaction'));
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request, $id)
    {
        $this->validate($request, [
            'sender_numAccount' => 'required|max:120',
            'receiver_numAccount' => 'required|max:120',
            'transaction_amount' => 'required',
        ]);

        
        $code1 = $request->input('sender_numAccount');
        $code2 = $request->input('receiver_numAccount');

        $account1 = Account::where('num_account', '=', $code1)
                            ->first();

        $account2 = Account::where('num_account', '=', $code2)
                            ->first();

        $transaction = Transaction::findOrFail($id);
        
        $old_amount = $transaction->transaction_amount;

        $transaction->sender_id = $account1->owner_id;
        $transaction->sender_accountId = $account1->id;
        $transaction->sender_numAccount = $request->input('sender_numAccount');
        $transaction->sender_name = $account1->owner_name;
        $transaction->sender_firstname = $account1->owner_firstname;
        $transaction->sender_email = $account1->owner_email;
        $transaction->sender_phone = $account1->owner_phone;
        $transaction->sender_address = $account1->owner_address;
        $transaction->sender_profession = $account1->owner_profession;

        $transaction->receiver_id = $account2->owner_id;
        $transaction->receiver_accountId = $account2->id;
        $transaction->receiver_numAccount = $request->input('receiver_numAccount');
        $transaction->receiver_name = $account2->owner_name;
        $transaction->receiver_firstname = $account2->owner_firstname;
        $transaction->receiver_email = $account2->owner_email;
        $transaction->receiver_phone = $account2->owner_phone;
        $transaction->receiver_address = $account2->owner_address;
        $transaction->receiver_profession = $account2->owner_profession;

        $transaction->transaction_amount = $request->input('transaction_amount');
        $transaction->status = 0;
        $transaction->user_id = auth()->user()->id;

        $account_receiver = Account::findOrFail($transaction->receiver_accountId);
        $balance = $account_receiver->balance - $old_amount;
        $account_receiver->balance = $balance + $transaction->transaction_amount;
        $account_receiver->save();

        $account_sender = Account::findOrFail($transaction->sender_accountId);
        $balance = $account_sender->balance + $old_amount;
        $account_sender->balance = $balance - $transaction->transaction_amount;
        $account_sender->save();

        $historique = new Historique();
        $historique->action = 'Uptate';
        $historique->table = 'TRANSACTION';
        $historique->user_id = auth()->user()->id;
        $transaction->save();
        $historique->save();

        //Redirect to the accounts.index view and display message
        return redirect()->route('transactions.index')
            ->with('success',
            'Transaction '.$transaction->code_transaction.' édité');
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function destroy($id)
    {
        $transaction = Transaction::findOrFail($id);
        $transaction->delete();

        return redirect()->route('transactions.index')
            ->with('success',
             'Transaction supprimé avec succès');
    }
}
